@@ -193,7 +193,7 @@ class ModflowSfr2(Package):
     heading = '# Streamflow-Routing (SFR2) file for MODFLOW, generated by Flopy'
     default_value = -1.0E+10
 
-    def __init__(self, model, nstrm=2, nss=0, nsfrpar=0, nparseg=0, const=128390.4, dleak=0.0001, istcb1=50, istcb2=66,
+    def __init__(self, model, nstrm=2, nss=1, nsfrpar=0, nparseg=0, const=128390.4, dleak=0.0001, istcb1=50, istcb2=66,
                  isfropt=0,
                  nstrail=10, isuzn=1, nsfrsets=30, irtflg=1, numtim=2, weight=0.75, flwtol=0.0001,
                  reach_data=None,
@@ -228,7 +228,11 @@ class ModflowSfr2(Package):
 
         # Dataset 1c. ----------------------------------------------------------------------
         self.nstrm = np.sign(nstrm) * len(reach_data) if reach_data is not None else nstrm  # number of reaches, negative value is flag for unsat. flow beneath streams and/or transient routing
-        self.nss = len(segment_data[0]) if segment_data is not None else nss  # number of stream segments
+        if segment_data is not None and not isinstance(segment_data, dict):
+            segment_data = {0: segment_data}
+        # use atleast_1d for length since segment_data might be a 0D array
+        # this seems to be OK, because self.segment_data is produced by the constructor (never 0D)
+        self.nss = len(np.atleast_1d(segment_data[0]))# number of stream segments
         self.nsfrpar = nsfrpar
         self.nparseg = nparseg
         self.const = const  # conversion factor used in calculating stream depth for stream reach (icalc = 1 or 2)
@@ -280,6 +284,11 @@ class ModflowSfr2(Package):
         self.stress_period_data = MfList(self, self.reach_data, dtype=self.reach_data.dtype)
 
         # Datasets 4 and 6. -----------------------------------------------------------------------
+
+        # list of values that indicate segments outside of the model
+        # (depending on how SFR package was constructed)
+        self.not_a_segment_values = [999999]
+
         self.segment_data = {0: self.get_empty_segment_data(self.nss)}
         if segment_data is not None:
             for i in segment_data.keys():
@@ -290,6 +299,9 @@ class ModflowSfr2(Package):
         if len(self.segment_data[0]) == 1 or \
                                 np.diff(self.segment_data[0].nseg).max() != 0 and np.diff(
                     self.segment_data[0].outseg).max() != 0:
+            # first convert any not_a_segment_values to 0
+            for v in self.not_a_segment_values:
+                self.segment_data[0].outseg[self.segment_data[0].outseg == v] = 0
             self.get_outreaches()
         self.channel_geometry_data = channel_geometry_data
         self.channel_flow_data = channel_flow_data
@@ -688,7 +700,8 @@ class ModflowSfr2(Package):
         last_reaches = np.append((np.diff(reach_data.iseg) == 1), True)
         reach_data.outreach = np.append(reach_data.reachID[1:], 0)
         # for now, treat lakes (negative outseg number) the same as outlets
-        reach_data.outreach[last_reaches] = [first_reaches.reachID[s] if s > 0 else 0
+        reach_data.outreach[last_reaches] = [first_reaches.reachID[s] if s > 0
+                                             else 0
                                              for s in segment_data.outseg - 1]
         self.reach_data['outreach'] = reach_data.outreach
 
@@ -720,8 +733,10 @@ class ModflowSfr2(Package):
         all_upsegs : dict
             Nested dictionary of form {stress period: {segment: [list of upsegs]}}
 
-        Note:
+        Notes
+        -----
         This method will not work if there are instances of circular routing.
+
         """
         all_upsegs = {}
         for per in range(self.nper):
@@ -926,7 +941,7 @@ class ModflowSfr2(Package):
 
         f_sfr.write(' '.join(fmts[0:4]).format(nseg, icalc, outseg, iupseg) + ' ')
 
-        if iupseg != 0:
+        if iupseg > 0:
             f_sfr.write(fmts[4].format(iprior) + ' ')
         if icalc == 4:
             f_sfr.write(fmts[5].format(nstrpts) + ' ')
@@ -1709,9 +1724,10 @@ def _get_item2_names(nstrm, reachinput, isfropt, structured=False):
         List of names (same as variables in SFR Package input instructions) of columns
         to assign (upon load) or retain (upon write) in reach_data array.
 
-    Note
-    ----
+    Notes
+    -----
     Lowercase is used for all variable names.
+
     """
     names = []
     if structured:
