@@ -13,6 +13,40 @@ from ..pakbase import Package
 from ..utils import Util2d, Util3d, read1d
 import warnings
 
+def read1d_fixed(f, a, nvalperline=8):
+    """
+    Read a 10 real value array (such as TIMPRS) and return it
+    as a.
+
+    Parameters
+    ----------
+    f : file handle
+    a : np.ndarray
+        array to fill
+
+    Returns
+    -------
+    a : np.ndarray
+
+    """
+    done = False
+    icount = 0
+    while True:
+        line = f.readline()
+        istart = 0
+        istop = 10
+        for i in range(nvalperline):
+            a[icount] = float(line[istart: istop])
+            istart = istop
+            istop += 10
+            if icount == a.shape[0] - 1:
+                done = True
+                break
+            icount += 1
+        if done:
+            break
+    return
+
 class Mt3dBtn(Package):
     """
     Basic Transport Package Class.
@@ -93,7 +127,8 @@ class Mt3dBtn(Package):
     obs: array of int
         An array with the cell indices (layer, row, column) for which the
         concentration is to be printed at every transport step. (default is
-        None).
+        None).  obs indices must be entered as zero-based numbers as a 1 is
+        added to them before writing to the btn file.
     nprobs: int
         An integer indicating how frequently the concentration at the specified
         observation points should be saved. (default is 1).
@@ -177,6 +212,11 @@ class Mt3dBtn(Package):
         self.nprs = nprs
         self.timprs = timprs
         self.obs = obs
+        if obs is not None:
+            if isinstance(obs, list):
+                obs = np.array(obs)
+            if obs.ndim != 2:
+                raise Exception('obs must be (or be convertible to) a 2d array')
         self.nprobs = nprobs
         self.chkmas = chkmas
         self.nprmas = nprmas
@@ -185,12 +225,15 @@ class Mt3dBtn(Package):
         self.species_names = species_names
         self.prsity = Util3d(model, (self.nlay, self.nrow, self.ncol),
                               np.float32, prsity, name='prsity',
-                              locat=self.unit_number[0])
+                              locat=self.unit_number[0],
+                              array_free_format=False)
         self.icbund = Util3d(model, (self.nlay, self.nrow, self.ncol), np.int,
                               icbund, name='icbund',
-                              locat=self.unit_number[0])
+                              locat=self.unit_number[0],
+                             array_free_format=False)
         self.ssflag = ssflag
-        self.dt0 = Util2d(model, (self.nper,), np.float32, dt0, name='dt0')
+        self.dt0 = Util2d(model, (self.nper,), np.float32, dt0, name='dt0',
+                          array_free_format=False)
         self.mxstrn = Util2d(model, (self.nper,), np.int, mxstrn,
                               name='mxstrn')
         self.ttsmult = Util2d(model, (self.nper,), np.float32, ttsmult,
@@ -201,7 +244,8 @@ class Mt3dBtn(Package):
         # Do some fancy stuff for multi-species concentrations
         self.sconc = []
         u3d = Util3d(model, (self.nlay, self.nrow, self.ncol), np.float32,
-                      sconc, name='sconc1', locat=self.unit_number[0])
+                     sconc, name='sconc1', locat=self.unit_number[0],
+                     array_free_format=False)
         self.sconc.append(u3d)
         if ncomp > 1:
             for icomp in range(2, ncomp + 1):
@@ -214,8 +258,9 @@ class Mt3dBtn(Package):
                           str(icomp) + " to zero, kwarg name " +
                           name)
                 u3d = Util3d(model, (self.nlay, self.nrow, self.ncol),
-                              np.float32, val, name=name,
-                              locat=self.unit_number[0])
+                             np.float32, val, name=name,
+                             locat=self.unit_number[0],
+                             array_free_format=False)
                 self.sconc.append(u3d)
 
         # Check to make sure that all kwargs have been consumed
@@ -229,13 +274,15 @@ class Mt3dBtn(Package):
 
     def setmodflowvars(self, nlay, nrow, ncol, nper, laycon, delr, delc, htop,
                             dz, perlen, nstp, tsmult):
-        '''
+        """
         Set these variables from the MODFLOW model, if it exists
-        '''
+
+        """
         # Members that may come from a modflow model (model.mf)
         validmfdis = True
         try:
             dum = self.parent.mf.dis.nlay
+            mf = self.parent.mf
         except:
             validmfdis = False
 
@@ -254,22 +301,22 @@ class Mt3dBtn(Package):
         if nlay is not None:
             self.nlay = nlay
         else:
-            self.nlay = self.parent.mf.dis.nlay
+            self.nlay = mf.dis.nlay
 
         if nrow is not None:
             self.nrow = nrow
         else:
-            self.nrow = self.parent.mf.dis.nrow
+            self.nrow = mf.dis.nrow
 
         if ncol is not None:
             self.ncol = ncol
         else:
-            self.ncol = self.parent.mf.dis.ncol
+            self.ncol = mf.dis.ncol
 
         if nper is not None:
             self.nper = nper
         else:
-            self.nper = self.parent.mf.dis.nper
+            self.nper = mf.dis.nper
 
         nlay = self.nlay
         nrow = self.nrow
@@ -279,12 +326,14 @@ class Mt3dBtn(Package):
         if delr is not None:
             self.delr = Util2d(self.parent, (ncol,), np.float32, delr,
                                 name='delr',
-                                locat=self.unit_number[0])
+                                locat=self.unit_number[0],
+                                array_free_format=False)
         else:
             self.delr = Util2d(self.parent, (ncol,), np.float32,
-                                self.parent.mf.dis.delr.get_value(),
+                                mf.dis.delr.get_value(),
                                 name='delr',
-                                locat=self.unit_number[0])
+                                locat=self.unit_number[0],
+                                array_free_format=False)
 
         if delc is not None:
             self.delc = Util2d(self.parent, (nrow,), np.float32, delc,
@@ -292,29 +341,34 @@ class Mt3dBtn(Package):
                                 locat=self.unit_number[0])
         else:
             self.delc = Util2d(self.parent, (nrow,), np.float32,
-                                self.parent.mf.dis.delc.get_value(),
+                                mf.dis.delc.get_value(),
                                 name='delc',
-                                locat=self.unit_number[0])
+                                locat=self.unit_number[0],
+                                array_free_format=False)
 
         if htop is not None:
             self.htop = Util2d(self.parent, (nrow, ncol), np.float32, htop,
                                 name='htop',
-                                locat=self.unit_number[0])
+                                locat=self.unit_number[0],
+                                array_free_format=False)
         else:
             self.htop = Util2d(self.parent, (nrow, ncol), np.float32,
-                                self.parent.mf.dis.top.get_value(),
+                                mf.dis.top.get_value(),
                                 name='htop',
-                                locat=self.unit_number[0])
+                                locat=self.unit_number[0],
+                                array_free_format=False)
 
         if dz is not None:
             self.dz = Util3d(self.parent, (nlay, nrow, ncol), np.float32, dz,
                               name='dz',
-                              locat=self.unit_number[0])
+                              locat=self.unit_number[0],
+                              array_free_format=False)
         else:
-            thickness = self.parent.mf.dis.thickness.get_value()
+            thickness = mf.dis.thickness.get_value()
             self.dz = Util3d(self.parent, (nlay, nrow, ncol), np.float32,
                               thickness, name='dz',
-                              locat=self.unit_number[0])
+                              locat=self.unit_number[0],
+                              array_free_format=False)
 
         if perlen is not None:
             self.perlen = Util2d(self.parent, (nper,), np.float32, perlen,
@@ -322,7 +376,7 @@ class Mt3dBtn(Package):
                                   locat=self.unit_number[0])
         else:
             self.perlen = Util2d(self.parent, (nper,), np.float32,
-                                  self.parent.mf.dis.perlen.get_value(),
+                                  mf.dis.perlen.get_value(),
                                   name='perlen',
                                   locat=self.unit_number[0])
 
@@ -332,7 +386,7 @@ class Mt3dBtn(Package):
                                 locat=self.unit_number[0])
         else:
             self.nstp = Util2d(self.parent, (nper,), np.int,
-                                self.parent.mf.dis.nstp.get_value(),
+                                mf.dis.nstp.get_value(),
                                 name='nstp',
                                 locat=self.unit_number[0])
 
@@ -342,7 +396,7 @@ class Mt3dBtn(Package):
                                   locat=self.unit_number[0])
         else:
             self.tsmult = Util2d(self.parent, (nper,), np.float32,
-                                  self.parent.mf.dis.tsmult.get_value(),
+                                  mf.dis.tsmult.get_value(),
                                   name='tsmult',
                                   locat=self.unit_number[0])
 
@@ -352,14 +406,14 @@ class Mt3dBtn(Package):
                                   name='laycon',
                                   locat=self.unit_number[0])
         else:
-            flow_package = self.parent.mf.get_package('BCF6')
+            flow_package = mf.get_package('BCF6')
             if flow_package is not None:
                 self.laycon = Util2d(self.parent, (nlay,), np.int,
                                       flow_package.laycon.get_value(),
                                       name='laycon',
                                       locat=self.unit_number[0])
             else:
-                flow_package = self.parent.mf.get_package('LPF')
+                flow_package = mf.get_package('LPF')
                 if flow_package is not None:
                     self.laycon = Util2d(self.parent, (nlay,),
                                           np.int, flow_package.laytyp.get_value(),
@@ -479,10 +533,10 @@ class Mt3dBtn(Package):
         else:
             nobs = self.obs.shape[0]
             f_btn.write('{0:10d}{1:10d}\n'.format(nobs, self.nprobs))
-            for r in range(nobs):
+            for i in range(nobs):
                 f_btn.write('{0:10d}{1:10d}{2:10d}\n' \
-                            .format(self.obs[r, 0], self.obs[r, 1],
-                                    self.obs[r, 2]))
+                            .format(self.obs[i, 0] + 1, self.obs[i, 1] + 1,
+                                    self.obs[i, 2] + 1))
 
         # A20 CHKMAS, NPRMAS
         if (self.chkmas == True):
@@ -509,6 +563,36 @@ class Mt3dBtn(Package):
 
     @staticmethod
     def load(f, model, ext_unit_dict=None):
+        """
+        Load an existing package.
+
+        Parameters
+        ----------
+        f : filename or file handle
+            File to load.
+        model : model object
+            The model object (of type :class:`flopy.mt3d.mt.Mt3dms`) to
+            which this package will be added.
+        ext_unit_dict : dictionary, optional
+            If the arrays in the file are specified using EXTERNAL,
+            or older style array control records, then `f` should be a file
+            handle.  In this case ext_unit_dict is required, which can be
+            constructed using the function
+            :class:`flopy.utils.mfreadnam.parsenamefile`.
+
+        Returns
+        -------
+        btn :  Mt3dBtn object
+            Mt3dBtn object.
+
+        Examples
+        --------
+
+        >>> import flopy
+        >>> mt = flopy.mt3d.Mt3dms()
+        >>> btn = flopy.mt3d.Mt3dBtn.load('test.btn', mt)
+
+        """
         if not hasattr(f, 'read'):
             filename = f
             f = open(filename, 'r')
@@ -576,42 +660,42 @@ class Mt3dBtn(Package):
         if model.verbose:
             print('   loading DELR...')
         delr = Util2d.load(f, model, (ncol, 1), np.float32, 'delr',
-                            ext_unit_dict)
+                            ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   DELR {}'.format(delr))
 
         if model.verbose:
             print('   loading DELC...')
         delc = Util2d.load(f, model, (nrow, 1), np.float32, 'delc',
-                            ext_unit_dict)
+                            ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   DELC {}'.format(delc))
 
         if model.verbose:
             print('   loading HTOP...')
         htop = Util2d.load(f, model, (nrow, ncol), np.float32, 'htop',
-                            ext_unit_dict)
+                            ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   HTOP {}'.format(htop))
 
         if model.verbose:
             print('   loading DZ...')
         dz = Util3d.load(f, model, (nlay, nrow, ncol), np.float32, 'dz',
-                          ext_unit_dict)
+                          ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   DZ {}'.format(dz))
 
         if model.verbose:
             print('   loading PRSITY...')
         prsity = Util3d.load(f, model, (nlay, nrow, ncol), np.float32, 'prsity',
-                              ext_unit_dict)
+                              ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   PRSITY {}'.format(prsity))
 
         if model.verbose:
             print('   loading ICBUND...')
         icbund = Util3d.load(f, model, (nlay, nrow, ncol), np.int, 'icbund',
-                              ext_unit_dict)
+                              ext_unit_dict, array_format="mt3d")
         if model.verbose:
             print('   ICBUND {}'.format(icbund))
 
@@ -619,14 +703,14 @@ class Mt3dBtn(Package):
             print('   loading SCONC...')
         kwargs = {}
         sconc = Util3d.load(f, model, (nlay, nrow, ncol), np.float32, 'sconc1',
-                             ext_unit_dict)
+                             ext_unit_dict, array_format="mt3d")
         if ncomp > 1:
             for icomp in range(2, ncomp + 1):
                 name = "sconc" + str(icomp)
                 if model.verbose:
                     print('   loading {}...'.format(name))
                 u3d = Util3d.load(f, model, (nlay, nrow, ncol), np.float32,
-                                   name, ext_unit_dict)
+                                   name, ext_unit_dict, array_format="mt3d")
                 kwargs[name] = u3d
         if model.verbose:
             print('   SCONC {}'.format(sconc))
@@ -672,7 +756,7 @@ class Mt3dBtn(Package):
             if model.verbose:
                 print('   loading TIMPRS...')
             timprs = np.empty((nprs), dtype=np.float32)
-            read1d(f, timprs)
+            read1d_fixed(f, timprs)
             if model.verbose:
                 print('   TIMPRS {}'.format(timprs))
 
